import ctypes
from ctypes import wintypes

# 定义回调函数类型
EnumWindowsProc = ctypes.WINFUNCTYPE(ctypes.c_bool, wintypes.HWND, wintypes.LPARAM)

# 定义 Windows API 函数
EnumWindows = ctypes.windll.user32.EnumWindows
GetWindowTextLength = ctypes.windll.user32.GetWindowTextLengthW
GetWindowText = ctypes.windll.user32.GetWindowTextW
SetForegroundWindow = ctypes.windll.user32.SetForegroundWindow

class WindowFinder:
    @staticmethod
    def find_window_by_title(title: str):
        hwnd_list = []

        def enum_windows_proc(hwnd: wintypes.HWND, lParam: wintypes.LPARAM):
            length = GetWindowTextLength(hwnd)
            if length > 0:
                window_title = ctypes.create_unicode_buffer(length + 1)
                GetWindowText(hwnd, window_title, length + 1)
                if title in window_title.value:
                    hwnd_list.append(hwnd)
            return True

        EnumWindows(EnumWindowsProc(enum_windows_proc), 0)
        return hwnd_list


from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By


class ElementUtils:
    @staticmethod
    def local_expression(name, control_type=None, class_name = None, automation_id = None, parent_name=None, parent_control_type=None):
        exp = ElementUtils.global_expression(name, control_type, class_name, automation_id, parent_name, parent_control_type)
        return f".{exp}"

    @staticmethod
    def global_expression(name, control_type=None, class_name = None, automation_id = None, parent_name=None, parent_control_type=None):
        exp = ""
        if parent_name:
            parent_tag_name = f"{parent_control_type.__value}" if parent_control_type is not None else "*"
            exp = f"{exp}//{parent_tag_name}[@Name='{parent_name}']"

        tag_name = f"{control_type.code}" if control_type is not None else "*"
        exp = f"{exp}//{tag_name}"

        name_str = None
        if name:
            name_str = f"@Name='{name}'"
        if class_name:
            name_str = f"{name_str} and @ClassName='{class_name}'" if name_str else f"@ClassName='{class_name}'"
        if automation_id:
            name_str = f"{name_str} and @AutomationId='{automation_id}'" if name_str else f"@AutomationId='{automation_id}'"

        if name_str:
            exp = f"{exp}[{name_str}]"

        return exp

    @staticmethod
    def find_element_by_type(element, name, control_type=None, parent_name=None, parent_control_type=None):
        exp = ElementUtils.local_expression(name, control_type, None, None, parent_name, parent_control_type)
        return element.find_element(By.XPATH, exp)

    @staticmethod
    def find_element_by_class(element, name, class_name=None, parent_name=None, parent_control_type=None):
        exp = ElementUtils.local_expression(name, None, class_name, None, parent_name, parent_control_type)
        return element.find_element(By.XPATH, exp)

    @staticmethod
    def find_element_by_automation(element, automation_id):
        exp = ElementUtils.local_expression(None, None, None, automation_id, None, None)
        return element.find_element(By.XPATH, exp)

    @staticmethod
    def find_elements_by_type(element, name, control_type=None, parent_name=None, parent_control_type=None):
        exp = ElementUtils.local_expression(name, control_type, None, None, parent_name, parent_control_type)
        return element.find_elements(By.XPATH, exp)

    @staticmethod
    def find_elements_by_class(element, name, class_name=None, parent_name=None, parent_control_type=None):
        exp = ElementUtils.local_expression(name, None, class_name, None, parent_name, parent_control_type)
        return element.find_elements(By.XPATH, exp)

    @staticmethod
    def find_element_by_types(element, name, control_types, parent_name=None, parent_control_type=None):
        ele = None
        for control_type in control_types:
            try:
                ele = ElementUtils.find_element_by_type(element, name, control_type, parent_name, parent_control_type)
            except NoSuchElementException:
                pass
        return ele




from mag_tools.model.base_enum import BaseEnum


class DataType(BaseEnum):
    NONE = (0, "无类型")  #: 无类型
    INTEGER = (1, "整数类型")  #: 整数类型
    FLOAT = (2, "浮点数类型")  #: 浮点数类型
    STRING = (3, "字符串类型")  #: 字符串类型
    BOOLEAN = (4, "布尔类型")  #: 布尔类型
    LIST = (5, "列表类型")  #: 列表类型
    DICTIONARY = (6, "字典类型")  #: 字典类型

    @classmethod
    def get_type(cls, value):
        """
        根据值的类型返回对应的 DataType 枚举类型。

        :param value: 要判断类型的值
        :return: 对应的 DataType 枚举类型
        """
        _type = cls.STRING  # 默认类型为 STRING

        if value is None:
            _type = cls.NONE
        elif isinstance(value, bool):
            _type = cls.BOOLEAN
        elif isinstance(value, int):
            _type = cls.INTEGER
        elif isinstance(value, float):
            _type = cls.FLOAT
        elif isinstance(value, str):
            if value.startswith('[') and value.endswith(']'):
                _type = cls.LIST
            elif value.startswith('{') and value.endswith('}'):
                _type = cls.DICTIONARY

        return _type


if __name__ == '__main__':
    # 示例用法
    print(DataType.INTEGER.code)  # 输出: 1
    print(DataType.INTEGER.desc)  # 输出: 整数类型
    print(DataType.get_by_code(1).desc)  # 输出: 整数类型
    print(DataType.get_type(123))  # 输出: DataType.INTEGER
